/* Generated by wayland-scanner 1.24.0 */

/*
 * SPDX-FileCopyrightText: 2019 Aleix Pol Gonzalez <aleixpol@kde.org>
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"


static const struct wl_interface *keystate_types[] = {
	NULL,
	NULL,
};

static const struct wl_message org_kde_kwin_keystate_requests[] = {
	{ "fetchStates", "", keystate_types + 0 },
	{ "destroy", "4", keystate_types + 0 },
};

static const struct wl_message org_kde_kwin_keystate_events[] = {
	{ "stateChanged", "uu", keystate_types + 0 },
};

WL_EXPORT const struct wl_interface org_kde_kwin_keystate_interface = {
	"org_kde_kwin_keystate", 5,
	2, org_kde_kwin_keystate_requests,
	1, org_kde_kwin_keystate_events,
};

